; Script to read radar timeseries and set up hypothesis 
; tests and their visualizations.
;
; Starting with Ho2: Reflectivity in the unblocked layer is not associated with 
; location 
;
;----------------------------;;----------------------------------->
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
;----------------------------;;----------------------------------->
begin
  err = NhlGetErrorObjectId()
  setvalues err
    "errLevel" : "Fatal"          ; only report Fatal errors
  end setvalues


;   wktype = "png"
;   wktype@wkHeight = 3000
;   wktype@wkWidth = 3000

   wktype = "x11"
   wktype@wkHeight = 1000
   wktype@wkWidth = 1000


optStat = True
optStat@PrintStat = True

fils = systemfunc ("ls ./PSD-Met-BBY*.nc")
a    = addfiles(fils, "r")
ListSetType (a, "cat")

time_base = a[:]->Times
nbasetim = dimsizes(time_base)
tunits = time_base@units

;-----------------------------------;;------------------------------------->
; Define Epoch boundaries for later plotting
;-----------------------------------;;------------------------------------->
  earlyar_st = tofloat(cd_inv_calendar(2016,03,05,15,5,0,tunits,0))
  earlyar_ed = tofloat(cd_inv_calendar(2016,03,05,20,55,0,tunits,0))
  
  barrjet_st = tofloat(cd_inv_calendar(2016,03,05,21,5,0,tunits,0))
  barrjet_ed = tofloat(cd_inv_calendar(2016,03,05,23,55,0,tunits,0))
  
  peakar_st = tofloat(cd_inv_calendar(2016,03,06,00,5,0,tunits,0))
  peakar_ed = tofloat(cd_inv_calendar(2016,03,06,02,55,0,tunits,0))
  
  postcf_st = tofloat(cd_inv_calendar(2016,03,06,03,5,0,tunits,0))
  postcf_ed = tofloat(cd_inv_calendar(2016,03,06,05,55,0,tunits,0))

  radar_analysis_start = tofloat(cd_inv_calendar(2016,03,05,11,55,0,tunits,0))
  radar_analysis_end   = tofloat(cd_inv_calendar(2016,03,06,05,55,0,tunits,0))


;--------------------------------------------->
; declare sonde data

ttop = (/-5.9,-4.9,-5.4,-4.2,-4.4,-4.4,-4.5,-4.9,-7.5,-9.2/)
tbot = (/-2.6,-0.9,-1.6,-1.2,-2.1,0.5,0.8,-1.0,-3.7,-5.7/)
sonde_time = "2016030"+(/"50547","51504","51826","52022","52217","60050","60220","60332","60516","60614"/)

n_sonde_tims = dimsizes(ttop)



year = tofloat(str_get_cols(sonde_time,0,3))
month = tofloat(str_get_cols(sonde_time,4,5))
day = tofloat(str_get_cols(sonde_time,6,7))
hour = tofloat(str_get_cols(sonde_time,8,9))
minute = tofloat(str_get_cols(sonde_time,10,11))
second = new((/n_sonde_tims/),float)
second(:) = 0.

snd_time_float = tofloat(cd_inv_calendar(year,month,day,hour,minute,second,tunits,0))

delete([/year,month,day,hour,minute,second/])
;print(""+rdr_time_float)


;**************************************************************
; Define own tick mark labels.
;**************************************************************
   
  
   xval             = tofloat(ispan(toint(min(time_base)),toint(max(time_base)),180))
   xval@units = time_base@units
   ;print(""+cd_calendar(xval,-3))
   
   
   xval2            = tofloat(ispan(toint(min(time_base)),toint(max(time_base)),60))                           
   xstring          = new((/dimsizes(xval)/),string)
   xstring(:)       = " "                                    
   do ix = 0,dimsizes(xval)-1                                    
    xstring(ix)          = "~F25~Mar-"+str_get_cols(cd_calendar(xval(ix),-3),6,7)+"~C~ @ "+str_get_cols(cd_calendar(xval(ix),-3),8,9)
   end do 
   ;print(""+xstring)

;----------------------------;;----------------------------------->
; read radar hydrometeor analyses
;----------------------------;;----------------------------------->

filename = "./test_hydrometeor_class.txt"

x1 = asciiread(filename,-1,"string")
nlines = dimsizes(x1)

x2 = x1(1:nlines-1)
nscans = nlines-1


prBBYrainCZCsnow = new((/nscans/),float)
prBBYrainCZCsnow@_FillValue = -9999.
prBBYsnowCZCsnow = new((/nscans/),float)
prBBYsnowCZCsnow@_FillValue = -9999.
prBBYprecCZCsnow = new((/nscans/),float)
prBBYprecCZCsnow@_FillValue = -9999.

prBBYrainCZCrain = new((/nscans/),float)
prBBYrainCZCrain@_FillValue = -9999.
prBBYsnowCZCrain = new((/nscans/),float)
prBBYsnowCZCrain@_FillValue = -9999.
prBBYprecCZCrain = new((/nscans/),float)
prBBYprecCZCrain@_FillValue = -9999.



timein = new((/nscans/),string)
radar_time = new((/nscans/),float)

do iline = 0,nscans-1
    temp = str_split(x2(iline), ",")
    timein(iline) = str_get_cols(temp(0),1,13)

     prBBYrainCZCsnow(iline) = tofloat(temp(1))
     prBBYsnowCZCsnow(iline) = tofloat(temp(2))
     prBBYprecCZCsnow(iline) = tofloat(temp(3))
     
     prBBYrainCZCrain(iline) = tofloat(temp(4))
     prBBYsnowCZCrain(iline) = tofloat(temp(5))
     prBBYprecCZCrain(iline) = tofloat(temp(6)) 
     
     
     year = tofloat(str_get_cols(timein(iline),0,3))
     month = tofloat(str_get_cols(timein(iline),4,5))
     day = tofloat(str_get_cols(timein(iline),6,7))
     hour = tofloat(str_get_cols(timein(iline),9,10))
     minute = tofloat(str_get_cols(timein(iline),11,12))
     radar_time(iline) = dble2flt(cd_inv_calendar(year,month,day,hour,minute,0,tunits,0))
     
end do

delete([/year,month,day,hour,minute,nlines,filename,timein,x1,x2,temp/])



;--------------------------------;;------------------------------------------->
; Derive the following timeseries: CZC rain, CZC snow, and CZC precipitating
;--------------------------------;;------------------------------------------->

isCZCsnow = new((/nscans/),integer)
isCZCrain = new((/nscans/),integer)
isCZCprec = new((/nscans/),integer)


isCZCsnow = where(.not.ismissing(prBBYsnowCZCsnow),1,0)
isCZCrain = where(.not.ismissing(prBBYsnowCZCrain),1,0)

isCZCprec = where(.not.ismissing(prBBYsnowCZCsnow).or..not.ismissing(prBBYsnowCZCrain),1,0)

isCZCprec = where(radar_time.lt.radar_analysis_start.or.radar_time.gt.radar_analysis_end,isCZCprec@_FillValue,isCZCprec)
isCZCrain = where(radar_time.lt.radar_analysis_start.or.radar_time.gt.radar_analysis_end,isCZCrain@_FillValue,isCZCrain)
isCZCsnow = where(radar_time.lt.radar_analysis_start.or.radar_time.gt.radar_analysis_end,isCZCsnow@_FillValue,isCZCsnow)

;; Additionally exclude Peak AR period

isCZCprec = where(radar_time.gt.peakar_st.and.radar_time.lt.peakar_ed,isCZCprec@_FillValue,isCZCprec)
isCZCrain = where(radar_time.gt.peakar_st.and.radar_time.lt.peakar_ed,isCZCrain@_FillValue,isCZCrain)
isCZCsnow = where(radar_time.gt.peakar_st.and.radar_time.lt.peakar_ed,isCZCsnow@_FillValue,isCZCsnow)

print("number CZC snow = "+num(isCZCsnow.eq.1))
print("number CZC rain = "+num(isCZCrain.eq.1))
print("number CZC prec = "+num(isCZCprec.eq.1))
print("prob.  CZC snow = "+(tofloat(num(isCZCsnow.eq.1))/tofloat(num(isCZCprec.eq.1))))


;--------------------------------;;------------------------------------------->
; Derive prBBYsnowCZCprec. Afterward, will have an estimate of the probability
; of snow from 44 gates for some number scans (number CZC prec).
; I will not know the order of BBY gates, but I can reconstruct how many total
; snow responses and how many total responses of either type this constitutes.
; This should be enough information to peform a one-tailed z-test or chi-square
; independence test.
; 
; To visualize: box-n-whisker? No. Timeseries works better. 
;--------------------------------;;------------------------------------------->


;stats = stat_dispersion(prBBYsnowCZCsnow,optStat)

;stats = stat_dispersion(prBBYsnowCZCrain,optStat)

prBBYsnowCZCprec = new((/nscans/),float)
prBBYsnowCZCprec@_FillValue = -9999.


ldx = ind(.not.ismissing(prBBYsnowCZCsnow))
mdx = ind(.not.ismissing(prBBYsnowCZCrain))

prBBYsnowCZCprec(ldx) = prBBYsnowCZCsnow(ldx)
prBBYsnowCZCprec(mdx) = prBBYsnowCZCrain(mdx)




;-----------------------------------;;------------------------------------->>
; Write test file to verify that prBBYsnowCZCprec was correctly constructed.
;-----------------------------------;;------------------------------------->>

fname = "test_BBY_probs_construction.txt"

header = (/"---------------------------------------------------------", \
           "   prBBYsnowCZCsnow   prBBYsnowCZCrain   prBBYsnowCZCprec", \
           "---------------------------------------------------------"/)
           
hlist = [/header/]
alist = [/prBBYsnowCZCsnow, prBBYsnowCZCrain, prBBYsnowCZCprec/]
           
;write_table(fname, "w", hlist, "%s")
;write_table(fname, "a", alist, "%19.2f%19.2f%19.2f")


prBBYsnowCZCprec = where(radar_time.lt.radar_analysis_start.or.radar_time.gt.radar_analysis_end,prBBYsnowCZCprec@_FillValue,prBBYsnowCZCprec)
prBBYsnowCZCprec = where(radar_time.gt.peakar_st.and.radar_time.lt.peakar_ed,prBBYsnowCZCprec@_FillValue,prBBYsnowCZCprec)

;----------------------------;;----------------------------------->
; read radar uncorrected reflectivity for BBY
;----------------------------;;----------------------------------->

filename = "./reflectivity_bby.txt"

x1 = asciiread(filename,-1,"string")
nlines = dimsizes(x1)

x2 = x1(1:nlines-1)
nscans = nlines-1
ngates = 45

dbBBY=new((/nscans,ngates/),float)
dbBBY@_FillValue = -9999.

do iline = 0,nscans-1

    temp = str_split(x2(iline), ",")

     do igate = 0,ngates-1
      dbBBY(iline,igate) = tofloat(temp(igate))
     end do 
    
end do    

dbBBY = where(dbBBY.le.-33.,dbBBY@_FillValue,dbBBY)

delete([/nlines,filename,x1,x2,temp/])

do igate = 0,ngates-1
 dbBBY(:,igate) = where(radar_time.lt.radar_analysis_start.or.radar_time.gt.radar_analysis_end,dbBBY@_FillValue,dbBBY(:,igate))
; dbCZC(:,igate) = where(radar_time.lt.radar_analysis_start.or.radar_time.gt.radar_analysis_end,dbCZC@_FillValue,dbCZC(:,igate))
end do 



;---------------------------;;------------------------------------------>>
; Calculate the needed entries for a chi-square independence test
; Note, NCL does not provide chi-square functions, so the analysis will
; be performed offline in an excel spreadsheet.
;---------------------------;;------------------------------------------>>

print("number scans with BBY information = "+num(.not.ismissing(prBBYsnowCZCprec)))
print("relevant number BBY gates is then:  "+(44*num(.not.ismissing(prBBYsnowCZCprec))))
print("number of those gates with snow is:  "+sum(44*prBBYsnowCZCprec))
print("mean #gates with BBY snow from above sample = "+avg(prBBYsnowCZCprec))


;;--------------------------------------;;----------------------------------->>
; The above counts yield the following Chi-square independence test tables:
; Observed  |  CZC  |  BBY
; Snow         75      888
; Rain         47     4480
;
; Expected  |  CZC  |  BBY
; Snow        21.4    941.6 
; Rain       100.6   4424.6 
;
; Conditional Probabilities:
pBBYsnow = 0.165
pCZCsnow = 0.615
;
; ChiSQ statistic:
X2 =  166.51 ;; df = 1
pX2 = 4.283e-38

exit


;-----------------------------------;;------------------------------------->>
; Create timeseries of prBBYsnowCZCprec, with polyline appended 
; representing the mean prCZCsnowCZCsnow.
;-----------------------------------;;------------------------------------->>

  wks = gsn_open_wks(wktype,"KDAX-ProbSnow-TS-Analysis")
  ;drawNDCGrid(wks)
  
    res = True
    res@gsnDraw     = False
    res@gsnFrame    = False
    
    res@vpXF                 = 0.15       				; x location
    res@vpYF                 = 0.92       				; y location
    res@vpWidthF             = 0.62       				; plot width
    res@vpHeightF            = 0.20       				; plot height
    
    res@gsnLeftString	   	   = "~F25~Fraction of BBY gates with Snow in each KDAX scan"
    res@gsnLeftStringFontHeightF     = 0.016
    res@gsnLeftStringParallelPosF    = 0.04
    res@gsnLeftStringOrthogonalPosF  = 0.04
  
    res@tmXTOn         = False
    res@tmYROn         = False
    res@tiXAxisFontHeightF = 0.015
    
  
    res@tmXBMode       = "Explicit" ;  
    res@tmXBValues     = xval
    res@tmXBMinorValues = xval2
    res@tmXBLabels     = xstring
    res@tmXBLabelFontHeightF = 0.013   
    
    
    res@trYMaxF = 1.1
    res@trYMinF = -0.1
    res@trXMaxF = radar_analysis_end
    res@trXMinF = radar_analysis_start
    
    res@tmYLLabelFontHeightF = 0.014
    res@tmYLLabelFont  = 25
    ;res@tmXBOn         = False

    res@tiYAxisString  = "~F25~Fraction BBY ~C~Gates w/ Snow"
    res@tiYAxisOffsetXF = -0.02
    res@tiYAxisFontHeightF = 0.014
    
    res@xyMarkLineMode = "Markers"
    res@xyMarker       = 4
    res@xyMarkerThicknessF = 6.0
    res@xyMarkerSizeF  = 0.015
    res@xyMarkerColor  = "black"
    res@xyCurveDrawOrder = "PostDraw"
    
    
    plot1 = gsn_csm_xy(wks,radar_time,prBBYsnowCZCprec,res)
    
    ;*************************************************************
    ; Use polylines to draw in Y ref lines. 
    
    line1x = (/ earlyar_st, earlyar_st /)
    liney = (/ -0.1, 1.1 /)
    
    line2x = (/earlyar_ed, earlyar_ed/)
    
    lnres = True
    lnres@gsLineColor = "green"
    lnres@gsLineDashPattern = 11
    lnres@gsLineThicknessF = 12.0 
  
    yref11 = gsn_add_polyline(wks,plot1,line1x,liney,lnres)
    yref12 = gsn_add_polyline(wks,plot1,line2x,liney,lnres)
    
    line1x = (/ barrjet_st, barrjet_st /)
    line2x = (/barrjet_ed, barrjet_ed/)
    
    lnres@gsLineColor = "gold2"
    yref13 = gsn_add_polyline(wks,plot1,line1x,liney,lnres)
    yref14 = gsn_add_polyline(wks,plot1,line2x,liney,lnres)
    
    line1x = (/ peakar_st, peakar_st /)
    line2x = (/peakar_ed, peakar_ed/)
    
    lnres@gsLineColor = "red"
    yref15 = gsn_add_polyline(wks,plot1,line1x,liney,lnres)
    yref16 = gsn_add_polyline(wks,plot1,line2x,liney,lnres)
    
    
    line1x = (/ postcf_st, postcf_st /)
    line2x = (/postcf_ed, postcf_ed/)
    
    lnres@gsLineColor = "purple"
    yref17 = gsn_add_polyline(wks,plot1,line1x,liney,lnres)
    yref18 = gsn_add_polyline(wks,plot1,line2x,liney,lnres)
    
    line1y = (/ tofloat(num(isCZCsnow.eq.1))/tofloat(num(isCZCprec.eq.1)), \
               tofloat(num(isCZCsnow.eq.1))/tofloat(num(isCZCprec.eq.1)) /)
    
    linex = (/radar_analysis_start, radar_analysis_end/)
    lnres@gsLineColor = "black"
    lnres@gsLineDashPattern = 3
    
    xref11 = gsn_add_polyline(wks,plot1,linex,line1y,lnres)
               
    
    

  draw(plot1)  
  
  
 ;-------------------------------;;------------------------------>
  ; Add ndc annotations
  ;-------------------------------;;------------------------------>
  
  txres = True
  txres@txFontColor = "black"
  txres@txFontHeightF = 0.018
  txres@txFontThicknessF = 3.0
  txres@txJust = "CenterLeft"
  
  gsn_text_ndc(wks,"~F25~Mean Prob. Snow @ CZC = "+pCZCsnow,0.12,0.61,txres)
  gsn_text_ndc(wks,"~F25~Mean Prob. Snow @ BBY = "+pBBYsnow,0.12,0.57,txres)
  gsn_text_ndc(wks,"~F25~Confidence That Prob. Snow is ~C~Independent of Site = "+pX2,0.12,0.51,txres)
  
  
    
  frame(wks) 
  delete(res)
  
  
end
 
 
 