; Script to read radar timeseries and set up hypothesis 
; tests and their visualizations.
;
; Starting with Ho2: Reflectivity in the unblocked layer is not associated with 
; location 
;
;----------------------------;;----------------------------------->
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
;----------------------------;;----------------------------------->
begin
  err = NhlGetErrorObjectId()
  setvalues err
    "errLevel" : "Fatal"          ; only report Fatal errors
  end setvalues


   wktype = "png"
   wktype@wkHeight = 3000
   wktype@wkWidth = 3000

;   wktype = "x11"
;   wktype@wkHeight = 1000
;   wktype@wkWidth = 1000


optStat = True
optStat@PrintStat = True

fils = systemfunc ("ls ./PSD-Met-BBY*.nc")
a    = addfiles(fils, "r")
ListSetType (a, "cat")

time_base = a[:]->Times
nbasetim = dimsizes(time_base)
tunits = time_base@units

;-----------------------------------;;------------------------------------->
; Define Epoch boundaries for later plotting
;-----------------------------------;;------------------------------------->
  earlyar_st = tofloat(cd_inv_calendar(2016,03,05,15,5,0,tunits,0))
  earlyar_ed = tofloat(cd_inv_calendar(2016,03,05,20,55,0,tunits,0))
  
  barrjet_st = tofloat(cd_inv_calendar(2016,03,05,21,5,0,tunits,0))
  barrjet_ed = tofloat(cd_inv_calendar(2016,03,05,23,55,0,tunits,0))
  
  peakar_st = tofloat(cd_inv_calendar(2016,03,06,00,5,0,tunits,0))
  peakar_ed = tofloat(cd_inv_calendar(2016,03,06,02,55,0,tunits,0))
  
  postcf_st = tofloat(cd_inv_calendar(2016,03,06,03,5,0,tunits,0))
  postcf_ed = tofloat(cd_inv_calendar(2016,03,06,05,55,0,tunits,0))

  radar_analysis_start = tofloat(cd_inv_calendar(2016,03,05,11,55,0,tunits,0))
  radar_analysis_end   = tofloat(cd_inv_calendar(2016,03,06,05,55,0,tunits,0))


;--------------------------------------------->
; declare sonde data

ttop = (/-5.9,-4.9,-5.4,-4.2,-4.4,-4.4,-4.5,-4.9,-7.5,-9.2/)
tbot = (/-2.6,-0.9,-1.6,-1.2,-2.1,0.5,0.8,-1.0,-3.7,-5.7/)
sonde_time = "2016030"+(/"50547","51504","51826","52022","52217","60050","60220","60332","60516","60614"/)

n_sonde_tims = dimsizes(ttop)



year = tofloat(str_get_cols(sonde_time,0,3))
month = tofloat(str_get_cols(sonde_time,4,5))
day = tofloat(str_get_cols(sonde_time,6,7))
hour = tofloat(str_get_cols(sonde_time,8,9))
minute = tofloat(str_get_cols(sonde_time,10,11))
second = new((/n_sonde_tims/),float)
second(:) = 0.

snd_time_float = tofloat(cd_inv_calendar(year,month,day,hour,minute,second,tunits,0))

delete([/year,month,day,hour,minute,second/])
;print(""+rdr_time_float)


;**************************************************************
; Define own tick mark labels.
;**************************************************************
   
  
   xval             = tofloat(ispan(toint(min(time_base)),toint(max(time_base)),180))
   xval@units = time_base@units
   ;print(""+cd_calendar(xval,-3))
   
   
   xval2            = tofloat(ispan(toint(min(time_base)),toint(max(time_base)),60))                           
   xstring          = new((/dimsizes(xval)/),string)
   xstring(:)       = " "                                    
   do ix = 0,dimsizes(xval)-1                                    
    xstring(ix)          = "~F25~Mar-"+str_get_cols(cd_calendar(xval(ix),-3),6,7)+"~C~ @ "+str_get_cols(cd_calendar(xval(ix),-3),8,9)
   end do 
   ;print(""+xstring)

;----------------------------;;----------------------------------->
; read radar hydrometeor analyses
;----------------------------;;----------------------------------->

filename = "./test_hydrometeor_class.txt"

x1 = asciiread(filename,-1,"string")
nlines = dimsizes(x1)

x2 = x1(1:nlines-1)
nscans = nlines-1


prBBYrainCZCsnow = new((/nscans/),float)
prBBYrainCZCsnow@_FillValue = -9999.
prBBYsnowCZCsnow = new((/nscans/),float)
prBBYsnowCZCsnow@_FillValue = -9999.
prBBYprecCZCsnow = new((/nscans/),float)
prBBYprecCZCsnow@_FillValue = -9999.

prBBYrainCZCrain = new((/nscans/),float)
prBBYrainCZCrain@_FillValue = -9999.
prBBYsnowCZCrain = new((/nscans/),float)
prBBYsnowCZCrain@_FillValue = -9999.
prBBYprecCZCrain = new((/nscans/),float)
prBBYprecCZCrain@_FillValue = -9999.



timein = new((/nscans/),string)
radar_time = new((/nscans/),float)

do iline = 0,nscans-1
    temp = str_split(x2(iline), ",")
    timein(iline) = str_get_cols(temp(0),1,13)

     prBBYrainCZCsnow(iline) = tofloat(temp(1))
     prBBYsnowCZCsnow(iline) = tofloat(temp(2))
     prBBYprecCZCsnow(iline) = tofloat(temp(3))
     
     prBBYrainCZCrain(iline) = tofloat(temp(4))
     prBBYsnowCZCrain(iline) = tofloat(temp(5))
     prBBYprecCZCrain(iline) = tofloat(temp(6)) 
     
     
     year = tofloat(str_get_cols(timein(iline),0,3))
     month = tofloat(str_get_cols(timein(iline),4,5))
     day = tofloat(str_get_cols(timein(iline),6,7))
     hour = tofloat(str_get_cols(timein(iline),9,10))
     minute = tofloat(str_get_cols(timein(iline),11,12))
     radar_time(iline) = dble2flt(cd_inv_calendar(year,month,day,hour,minute,0,tunits,0))
     
end do

delete([/year,month,day,hour,minute,nlines,filename,timein,x1,x2,temp/])



;--------------------------------;;------------------------------------------->
; Derive the following timeseries: CZC rain, CZC snow, and CZC precipitating
;--------------------------------;;------------------------------------------->

isCZCsnow = new((/nscans/),integer)
isCZCrain = new((/nscans/),integer)
isCZCprec = new((/nscans/),integer)


isCZCsnow = where(.not.ismissing(prBBYsnowCZCsnow),1,0)
isCZCrain = where(.not.ismissing(prBBYsnowCZCrain),1,0)

isCZCprec = where(.not.ismissing(prBBYsnowCZCsnow).or..not.ismissing(prBBYsnowCZCrain),1,0)

isCZCprec = where(radar_time.lt.radar_analysis_start.or.radar_time.gt.radar_analysis_end,isCZCprec@_FillValue,isCZCprec)
isCZCrain = where(radar_time.lt.radar_analysis_start.or.radar_time.gt.radar_analysis_end,isCZCrain@_FillValue,isCZCrain)
isCZCsnow = where(radar_time.lt.radar_analysis_start.or.radar_time.gt.radar_analysis_end,isCZCsnow@_FillValue,isCZCsnow)


print("number CZC snow = "+num(isCZCsnow.eq.1))
print("number CZC rain = "+num(isCZCrain.eq.1))
print("number CZC prec = "+num(isCZCprec.eq.1))
print("prob.  CZC snow = "+(tofloat(num(isCZCsnow.eq.1))/tofloat(num(isCZCprec.eq.1))))


;--------------------------------;;------------------------------------------->
; Derive prBBYsnowCZCprec. Afterward, will have an estimate of the probability
; of snow from 44 gates for some number scans (number CZC prec).
; I will not know the order of BBY gates, but I can reconstruct how many total
; snow responses and how many total responses of either type this constitutes.
; This should be enough information to peform a one-tailed z-test or chi-square
; independence test.
; 
; To visualize: box-n-whisker? No. Timeseries works better. 
;--------------------------------;;------------------------------------------->


;stats = stat_dispersion(prBBYsnowCZCsnow,optStat)

;stats = stat_dispersion(prBBYsnowCZCrain,optStat)

prBBYsnowCZCprec = new((/nscans/),float)
prBBYsnowCZCprec@_FillValue = -9999.


ldx = ind(.not.ismissing(prBBYsnowCZCsnow))
mdx = ind(.not.ismissing(prBBYsnowCZCrain))

prBBYsnowCZCprec(ldx) = prBBYsnowCZCsnow(ldx)
prBBYsnowCZCprec(mdx) = prBBYsnowCZCrain(mdx)


prBBYsnowCZCprec = where(radar_time.lt.radar_analysis_start.or.radar_time.gt.radar_analysis_end,prBBYsnowCZCprec@_FillValue,prBBYsnowCZCprec)


;----------------------------;;----------------------------------->
; read radar uncorrected reflectivity for BBY
;----------------------------;;----------------------------------->

filename = "./reflectivity_bby.txt"

x1 = asciiread(filename,-1,"string")
nlines = dimsizes(x1)

x2 = x1(1:nlines-1)
nscans = nlines-1
ngates = 45

dbBBY=new((/nscans,ngates/),float)
dbBBY@_FillValue = -9999.

do iline = 0,nscans-1

    temp = str_split(x2(iline), ",")

     do igate = 0,ngates-1
      dbBBY(iline,igate) = tofloat(temp(igate))
     end do 
    
end do    

dbBBY = where(dbBBY.le.-33.,dbBBY@_FillValue,dbBBY)

delete([/nlines,filename,x1,x2,temp/])

;----------------------------;;----------------------------------->
; read radar uncorrected reflectivity for CZC
;----------------------------;;----------------------------------->


filename = "./reflectivity_czc.txt"

x1 = asciiread(filename,-1,"string")
nlines = dimsizes(x1)

x2 = x1(1:nlines-1)
nscans = nlines-1

dbCZC=new((/nscans,ngates/),float)
dbCZC@_FillValue = -9999.

do iline = 0,nscans-1

    temp = str_split(x2(iline), ",")

     do igate = 0,ngates-1
      dbCZC(iline,igate) = tofloat(temp(igate))
     end do 
    
end do    

dbCZC = where(dbCZC.le.-33.,dbCZC@_FillValue,dbCZC)
;print(""+dimsizes(temp))
delete([/nlines,filename,x1,x2,temp/])


do igate = 0,ngates-1
 dbBBY(:,igate) = where(radar_time.lt.radar_analysis_start.or.radar_time.gt.radar_analysis_end,dbBBY@_FillValue,dbBBY(:,igate))
 dbCZC(:,igate) = where(radar_time.lt.radar_analysis_start.or.radar_time.gt.radar_analysis_end,dbCZC@_FillValue,dbCZC(:,igate))
end do 
  
  
;------------------------------------;;------------------------------------->
; Create BBY reflectivity to snow probability scatterplot
;
;------------------------------------;;------------------------------------->

  dbCZCscan = dbCZC(:,0)
  dbBBYscan = dim_avg_n(dbBBY,1)

  Xplot = ndtooned(prBBYsnowCZCprec)
  Xplot_time = radar_time
  Yplot = ndtooned(dbBBYscan)
  
  npair = dimsizes(Xplot)
  
  ;print("dimsizes X and Y are :"+npair+" and "+dimsizes(Yplot)+", respectively.")
  
;--------------------;;------------------------->
; Are prob snow and reflectivity correlated at lag 0?

rc = regline(Xplot,Yplot)
cor = escorc(Xplot,Yplot)
npair = num(.not.ismissing(Xplot)) 

df = npair-2 

t    = cor*sqrt((npair-2)/(1-cor^2))      
p    = student_t(t, df)
psig = 0.05                       ; test significance level                     
print("#deg. freedom = "+df)
print("t="+t+"  p="+p)          
if (p.le.psig) then
     print("cor="+cor+" is significant at the 95% level")
else
     print("cor="+cor+" is NOT significant at the 95% level")
end if  


;-----------------------------------;;-------------------------------------->
; Plot Reflectivity vs Snow Prob Scatter Diagram
;-----------------------------------;;-------------------------------------->
 

   
   
  wks = gsn_open_wks(wktype,"BBY-ProbSnow-v-MeanReflectivity")
  ;drawNDCGrid(wks)
  
    res = True
    res@gsnDraw     = False
    res@gsnFrame    = False
    
    res@vpXF                 = 0.16       				; x location
    res@vpYF                 = 0.92       				; y location
    res@vpWidthF             = 0.6       				; plot width
    res@vpHeightF            = 0.6       				; plot height

    res@trYMinF            =  0.               ; min value on y-axis
    res@trYMaxF            =  30. ; max value on y-axis
    res@trXMinF            =  0.                  ; min value on x-axis
    res@trXMaxF            =  1.15*max(Xplot)  ; max value on x-axis
  
    res@tmXTOn             = False
    res@tmYROn             = False
   
;---XY marker resources
  res@xyMarkLineMode     = "Markers"
  res@xyMarker           = 16
  res@xyMarkerSizeF      = 0.016
  res@xyMarkerThicknessF = 3.
  res@xyMarkerColor      = "white"
  
;---Tickmark resources
  res@tmXBMode         = "Manual"
  res@tmXBLabelFont    = 25  
  res@tmXBLabelFontHeightF = 0.021
  res@tmYLMode         = "Manual"
  res@tmYLLabelFont    = 25
  res@tmYLLabelFontHeightF = 0.021
  res@tmXBTickSpacingF = 0.2
  res@tmYLTickSpacingF = 5.

;---Title resources
  res@tiMainString       = "~F25~Relationship BBY Snow to BBY Reflectivity"
  res@tiXAxisString      = "~F25~BBY Chance Snow (%)"
  res@tiYAxisString      = "~F25~BBY Reflectivity (dBZ)"
  res@tiMainFontHeightF  = 0.020  


  plot1 = gsn_csm_xy (wks,Xplot, Yplot, res)


;----------------------------------;;--------------------------------->
; Add trendline to background plot
;----------------------------------;;---------------------------------> 

  
  xspan = fspan(0,1.15*max(Xplot),21)
  line = xspan*rc + rc@yintercept
  
  
  plres = True
  plres@gsLineDashPattern = 14
  plres@gsLineThicknessF  = 6.5
  plres@gsLineColor       = "blue"
  
  dum = gsn_add_polyline(wks, plot1, xspan, line, plres) 

;----------------------------------;;--------------------------------->
; Add reflectivity as polymarkers
;----------------------------------;;---------------------------------> 

dum1 = new((/nscans/),graphic)

pmres = True

pmres@gsMarkerIndex      = 16
pmres@gsMarkerSizeF      = 0.016
pmres@gsMarkerThicknessF = 6.
pmres@gsMarkerColor      = "black"

do imark = 0,nscans-1  ;; num colors in table

 if(Xplot_time(imark).ge.earlyar_st.and.Xplot_time(imark).le.earlyar_ed)then
  pmres@gsMarkerColor      = "green"
 end if
 if(Xplot_time(imark).ge.barrjet_st.and.Xplot_time(imark).le.barrjet_ed)then
  pmres@gsMarkerColor      = "gold2"
 end if
 if(Xplot_time(imark).ge.peakar_st.and.Xplot_time(imark).le.peakar_ed)then
  pmres@gsMarkerColor      = "red"
 end if
 if(Xplot_time(imark).ge.postcf_st.and.Xplot_time(imark).le.postcf_ed)then
  pmres@gsMarkerColor      = "purple"
 end if
 
  
 dum1(imark) = gsn_add_polymarker(wks, plot1, Xplot(imark), Yplot(imark), pmres)

end do 


    draw(plot1)

 

  
  ;-------------------------------;;------------------------------>
  ; Add ndc annotations
  ;-------------------------------;;------------------------------>
  
  txres = True
  txres@txFontColor = "black"
  txres@txFontHeightF = 0.018
  txres@txJust = "CenterLeft"
  txres@txFontThicknessF = 3.0
  
  
  pmres@gsMarkerThicknessF = 6.0
  pmres@gsMarkerSizeF  = 0.015
  
  
  pmres@gsMarkerColor  = "green"
  pmres@gsMarkerIndex       = 16
  gsn_text_ndc(wks,"~F25~Early-AR",0.83,0.63,txres)
  gsn_polymarker_ndc(wks,0.80,0.63,pmres)
  
  pmres@gsMarkerColor  = "gold2"
  pmres@gsMarkerIndex       = 16
  gsn_text_ndc(wks,"~F25~Barrier-Jet",0.83,0.60,txres)
  gsn_polymarker_ndc(wks,0.80,0.60,pmres)
  
  pmres@gsMarkerColor  = "red"
  pmres@gsMarkerIndex       = 16
  gsn_text_ndc(wks,"~F25~Peak-AR",0.83,0.57,txres)
  gsn_polymarker_ndc(wks,0.80,0.57,pmres)
  
  pmres@gsMarkerColor  = "purple"
  pmres@gsMarkerIndex       = 16
  gsn_text_ndc(wks,"~F25~Post-CF",0.83,0.54,txres)
  gsn_polymarker_ndc(wks,0.80,0.54,pmres)
  
  pmres@gsMarkerColor  = "black"
  pmres@gsMarkerIndex       = 16
  gsn_text_ndc(wks,"~F25~Other",0.83,0.51,txres)
  gsn_polymarker_ndc(wks,0.80,0.51,pmres)

  gsn_text_ndc(wks,"~F25~R~S~2~N~: "+sprintf("%5.4f",(cor*cor)),0.83,0.78,txres)
  gsn_text_ndc(wks,"~F25~T-Test ~C~Prob: "+sprintf("%5.4f",p),0.83,0.70,txres)
  
  
  
  frame(wks)
  delete(res)


;-------------------------------------;;------------------------------------->
; In workstation 2; test whether the CZC reflectivities are 
; Different when snowing vs. not snowing.
; 1) Perform significance test
;-------------------------------------;;------------------------------------->

dbCZCsnow = new((/nscans/),float)
dbCZCrain = new((/nscans/),float)

dbCZCsnow(ldx) = dbCZCscan(ldx)
dbCZCrain(mdx) = dbCZCscan(mdx)

print("number dbCZCsnow = "+num(.not.ismissing(dbCZCsnow)))
print("number dbCZCrain = "+num(.not.ismissing(dbCZCrain)))




;jgate = 44
;jgate = ispan(0,44,1)

Xtest = dbCZCrain
Ytest = dbCZCsnow

siglvl  = 0.05
aveX    = avg(Xtest) ;; dBZ during CZC rain
aveY    = avg(Ytest) ;; dBZ during CZC snow
varX    = variance(Xtest)
varY    = variance(Ytest)

sX      = num(.not.ismissing(Xtest))
sY      = num(.not.ismissing(Ytest))

;stats = stat_dispersion(Xtest,optStat)
;exit

print("CZC rain refl. variance = "+varX+" CZC snow refl. variance = "+varY)
print("CZC rain sample size = "+sX+" CZC snow sample size = "+sY)



iflag   = True
tval_opt  = False
prob = ttest(aveX,varX,sX, aveY,varY,sY, iflag, tval_opt)

print("t-test prob. = "+prob)
if (prob.le.siglvl) then
     print("difference in mean reflectivities of "+(aveX - aveY)+" is significant at the 95% level")
else
     print("difference in mean reflectivities of "+(aveX - aveY)+" is not significant at the 95% level")
end if  


;-----------------------------------;;-------------------------------------->
; Plot Reflectivity Box-and-whisker
;-----------------------------------;;-------------------------------------->

optStat@PrintStat = False

statsCZCrain = stat_dispersion(Xtest,optStat)
statsCZCsnow = stat_dispersion(Ytest,optStat)


yval = new((/2,5/),float)

yval(0,0) = statsCZCrain(2)
yval(0,1) = statsCZCrain(6)
yval(0,2) = statsCZCrain(8)
yval(0,3) = statsCZCrain(10)
yval(0,4) = statsCZCrain(14)

yval(1,0) = statsCZCsnow(2)
yval(1,1) = statsCZCsnow(6)
yval(1,2) = statsCZCsnow(8)
yval(1,3) = statsCZCsnow(10)
yval(1,4) = statsCZCsnow(14)


x = (/-2.25,2.25/)


;  wks2 = gsn_open_wks(wktype,"CZC-ReflBoxnWhisker-RainvSnow")


;**********************************************
; resources for plot background
;**********************************************
  res            = True 
  
  res@vpXF                 = 0.24       				; x location
    res@vpYF                 = 0.92       				; y location
    res@vpWidthF             = 0.6       				; plot width
    res@vpHeightF            = 0.6       				; plot height                        
  res@gsnDraw     = False
  res@gsnFrame    = False
  res@tmXBLabelFontHeightF = 0.045
  res@tmXBLabels = (/"~F25~Rain ","~F25~Snow "/) ; labels for each box
  res@tiMainString = " "
;**********************************************
; resources for polylines that draws the boxes
;**********************************************  
  llres                   = True			
  llres@gsLineThicknessF  = 2.5                 ; line thickness 
;**********************************************
; resources that control color and width of boxes
;**********************************************  
  opti          = True			
  opti@boxWidth = 2.5				; Width of box (x units)
  opti@boxColors = (/"blue","black"/)  	; Color of box(es)
;***********************************************
  res@gsnMaximize = True                        ; Maximize box plot in frame.
  res@tmYLMode    = "Explicit"
  res@tmYLLabelFontHeightF = 0.045
  res@tmYLValues  = (/0., 5., 10., 15., 20., 25., 30./)
  res@tmYLLabels  = (/"~F25~0", "~F25~5", "~F25~10", "~F25~15", "~F25~20", "~F25~25", "~F25~30"/)
;***********************************************

;  plot2 = boxplot(wks2,x,yval,opti,res,llres)	; All 3 options used...
;  draw(plot2)
;**********************************************
; adding additional text to the plot
;**********************************************  
  tres               = True                    ; text mods desired

  ; add text in ndc coordinates

  tres@txFontColor = "blue"
  tres@txFontHeightF = 0.045
  tres@txFont = 25
  ;gsn_text_ndc(wks2,"Rain~C~@ CZC",0.77,0.74, tres)

  tres@txFontColor = "black"
  ;gsn_text_ndc(wks2,"Snow~C~@ CZC",0.77,0.77, tres)

  tres@txFontHeightF = 0.015 
  tres@txFontColor = "black"
;  gsn_text_ndc(wks2,"T-Test Prob ~C~Refl. is Similar ~C~During Rain/Snow ~C~over CZC: ~C~"+prob,0.88,0.76, tres)

  tres@txFontHeightF = 0.045

  tres@txAngleF    = 90.
  
  tres@txFontColor = "black"
;  gsn_text_ndc(wks2,"~F25~dBZ",0.06,0.65, tres)

  print("t-test prob. = "+prob)

;frame(wks2)

   
 end
 
 
 