; IDL PROGRAM TO CALCULATE TIME SERIES OF HYDROMETEOR SPECIES AT BODEGA
; BAY AND CAZADERO FROM NEXRAD KDAX(SACRAMENTO) DATA 

; ALL LEVEL2 NEXRAD RADAR DATA WERE DOWNLOADED FROM AMAZON WEB
; SERVICES THROUGH THE NOAA WEATHER AND CLIMATE TOOLKIT AS NETCDF
; FILES ON 10 MARCH 2017
; https://www.ncdc.noaa.gov/wct/

.run /home/fcannon/routines/tvim.pro
.run /home/fcannon/routines/color_key.pro

; LIST THE DOWNLOADED LEVEL 2 HYDROMETEOR SPECIES DATA FOR THE EVENT
ncdf_list,'/data/downloaded/Observations/GPM/NEXRAD/3.6.16/DAX/N0H/KDAX_N0H_20160306_035000.nc',/variables
lists = file_search('/data/downloaded/Observations/GPM/NEXRAD/3.6.16/DAX/N0H/KDAX_N0H_*') 

; LOAD DATA
scans = 1290
elv = fltarr(360,scans)
lat = fltarr(360,scans)
lon = fltarr(360,scans)
azm = fltarr(360,scans)
alt = fltarr(360,scans)
tim = fltarr(360,scans)
gat = fltarr(1200,scans)
n0h = fltarr(1200,360,scans)
fnm = strarr(scans)
tx = 0
.run
for dd = 0,1 do begin
   day = dd+5
   for hh = 0,23 do begin
      if hh lt 10 then hour = strcompress(string('0',hh),/remove_all)
      if hh ge 10 then hour = strcompress(string(hh),/remove_all)
      for mm = 0,59 do begin
         if mm lt 10 then mint = strcompress(string('0',mm),/remove_all)
         if mm ge 10 then mint = strcompress(string(mm),/remove_all)
         
         directory = '/data/downloaded/Observations/GPM/NEXRAD/3.6.16/DAX/N0H/KDAX_N0H_2016030'
         datn = strcompress(string(directory,day,'_',hour,mint,'00.nc'),/remove_all) & datn = datn(0)+datn(1)
         filename = file_search(datn) 
         testy = file_test(filename)

            if testy(0) eq 1 then begin
            id = ncdf_open(filename)
            tht = ncdf_varid(id,'elevation')
            ncdf_varget,id,tht,elv1
            tht = ncdf_varid(id,'gate')
            ncdf_varget,id,tht,gat1
            tht = ncdf_varid(id,'latitude')
            ncdf_varget,id,tht,lat1
            tht = ncdf_varid(id,'longitude')
            ncdf_varget,id,tht,lon1
            tht = ncdf_varid(id,'azimuth')
            ncdf_varget,id,tht,azm1
            tht = ncdf_varid(id,'altitude')
            ncdf_varget,id,tht,alt1
            tht = ncdf_varid(id,'rays_time')
            ncdf_varget,id,tht,tim1
            ;tht = ncdf_varid(id,'DigitalInstantaneousPrecipitationRate')
            tht = ncdf_varid(id,'HydrometeorClassification')
            ncdf_varget,id,tht,n0h1

            elv(*,tx) = elv1
            lat(*,tx) = lat1
            lon(*,tx) = lon1
            azm(*,tx) = azm1
            alt(*,tx) = alt1
            tim(*,tx) = tim1
            gat(*,tx) = gat1
            n0h(*,*,tx) = n0h1
            fnm(tx) = filename

            tx = tx+1
         endif
         endfor
      print,hh
   endfor
endfor
end



; GIVE A LAT/LON COORDINATE TO EACH DATA POINT
clat = lat(0) & clon = lon(0)
numd = n_elements(gat(*,0)) & numa = n_elements(azm(*,0))

pointlat = fltarr(numd,numa,scans)
pointlon = fltarr(numd,numa,scans)
pointhgt = fltarr(numd,numa,scans)
pointdst = fltarr(numd,numa,scans)
.run
for nn = 0,scans-1 do begin
for zz = 0,numa-1 do begin
az = azm(zz,nn)
xx = gat(*,nn)*cos(!dtor*(elv(az,nn)))-((gat(*,nn)^2.)/(2*1.21*6371000.)) ; FROM WSR-88D PDF
yy = gat(*,nn)*sin(!dtor*(elv(az,nn)))+((gat(*,nn)^2.)/(2*1.21*6371000.))

lat1 = clat*(!pi/180.)
lon1 = clon*(!pi/180.)
brg = az*(!pi/180.)
radius = 6372797. ; earth
distrad = xx/radius
lat2 = asin(sin(lat1)*cos(distrad)+cos(lat1)*sin(distrad)*cos(brg)) 
lon2 = lon1+atan(sin(brg)*sin(distrad)*cos(lat1),cos(distrad)-sin(lat1)*sin(lat2))
lat2 = lat2*(180./!pi) & lon2 = lon2*(180./!pi)
pointlat(*,zz,nn) = lat2
pointlon(*,zz,nn) = lon2
pointhgt(*,zz,nn) = yy
pointdst(*,zz,nn) = gat(*,nn)
endfor
print,nn
endfor
end



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; PLOT COVERAGE
i.erase
i = map('geographic',limit=[clat-3.1,clon-3-.1,clat+3+.1,clon+3],/current,font_size=10,position=[.1,.2,.9,.8],/box_axes,grid_longitude=1,grid_latitude=1,linestyle=6,label_position=0)
   lonp = pointlon(*,*,708) & lonp = lonp(*)
   latp = pointlat(*,*,708) & latp = latp(*)
   aa = n0h(*,*,708) & aa = aa(*)
   p.delete
   p = scatterplot(lonp(where(aa eq aa)),latp(where(aa eq aa)),magnitude=aa(where(aa eq aa)),symbol='o',sym_filled=1,rgb_table=33,overplot=i,sym_size=.2)
                                ; 10 = Biological | 20 =  Clutter | 30
                                ; =  Ice Crystal | 40 = Dry Snow | 50
                                ; = Wet Snow | 60 = Lt./Mod Rain | 70
                                ; = Heavy Rain | 80 = Big Drops | 90 =
                                ; Graupel | 100 = Hail | 140 = Unknown
   
bb = dprp(uniq(dprp,sort(dprp)))   
   
pp = scatterplot([clon,clon],[clat,clat],/data,sym_filled=1,sym_size=1,overplot=i,symbol='o',sym_fill_color='white')
c = mapcontinents(/usa)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


; FIND CLOSEST TO CAZADERO
czclon = -123.2152
czclat = 38.6107

n0hczc = fltarr(scans)
elvczc = fltarr(scans)
dstczc = fltarr(scans)
.run
for nn = 0,scans-1 do begin
   dist = sqrt((pointlon(*,*,nn)-czclon)^2.+(pointlat(*,*,nn)-czclat)^2.)
   near = where(dist eq min(dist))
   tmpn = n0h(*,*,nn)
   tmpe = pointhgt(*,*,nn)
   id = array_indices(tmpn,near)
   n0hczc(nn) = tmpn(id(0),id(1),*)
   elvczc(nn) = tmpe(id(0),id(1),*)
   dstczc(nn) = pointdst(id(0),id(1),nn)
   print,nn
endfor
end



; FIND CLOSEST TO BODEGA
bbylon = -123.0728
bbylat = 38.3191
bbylon = -123.2116; north
bbylat = 38.4978
bbylon = -123.08; south
bbylat = 38.025

n0hbby = fltarr(scans)
elvbby = fltarr(scans)
dstbby = fltarr(scans)
.run
for nn = 0,scans-1 do begin
   dist = sqrt((pointlon(*,*,nn)-bbylon)^2.+(pointlat(*,*,nn)-bbylat)^2.)
   near = where(dist eq min(dist))
   tmpn = n0h(*,*,nn)
   tmpe = pointhgt(*,*,nn)
   id = array_indices(tmpn,near)
   n0hbby(nn) = tmpn(id(0),id(1),*)
   elvbby(nn) = tmpe(id(0),id(1),*)
   dstbby(nn) = pointdst(id(0),id(1),nn)
   print,nn
endfor
end






; FIND FULL RANGE GATE
czclon = -123.2152
czclat = 38.6107
czc_n0hray = fltarr(1200,scans)
czc_lonray = fltarr(1200,scans)
czc_latray = fltarr(1200,scans)
czc_hgtray = fltarr(1200,scans)
czc_dstray = fltarr(1200,scans)
.run
for nn = 0,scans-1 do begin ; for czc
   dist = sqrt((pointlon(*,*,nn)-czclon)^2.+(pointlat(*,*,nn)-czclat)^2.)
   near = where(dist eq min(dist))
   tmpe = pointhgt(*,*,nn)
   idc = array_indices(tmpe,near)
   czc_lonray(*,nn) = reform(pointlon(*,idc(1),nn))
   czc_latray(*,nn) = reform(pointlat(*,idc(1),nn))
   czc_hgtray(*,nn) = reform(pointhgt(*,idc(1),nn))
   czc_n0hray(*,nn) = reform(n0h(*,idc(1),nn))
   czc_dstray(*,nn) = reform(pointdst(*,idc(1),nn))
   print,nn
endfor
end

;bbylon = -123.0728
;bbylat = 38.3191
bby_n0hray = fltarr(1200,scans)
bby_lonray = fltarr(1200,scans)
bby_latray = fltarr(1200,scans)
bby_hgtray = fltarr(1200,scans)
bby_dstray = fltarr(1200,scans)
.run
for nn = 0,scans-1 do begin ; for bby
   dist = sqrt((pointlon(*,*,nn)-bbylon)^2.+(pointlat(*,*,nn)-bbylat)^2.)
   near = where(dist eq min(dist))
   tmpe = pointhgt(*,*,nn)
   idc = array_indices(tmpe,near)
   bby_lonray(*,nn) = reform(pointlon(*,idc(1),nn))
   bby_latray(*,nn) = reform(pointlat(*,idc(1),nn))
   bby_hgtray(*,nn) = reform(pointhgt(*,idc(1),nn))
   bby_n0hray(*,nn) = reform(n0h(*,idc(1),nn))
   bby_dstray(*,nn) = reform(pointdst(*,idc(1),nn))
   print,nn
endfor
end



; BBY DISTANCE TO HEIGHT
keep = where(bby_dstray(*,600) ge dstbby and bby_dstray(*,600) le dstczc,nk)
;keep = where(bby_dstray(*,600) ge dstczc and bby_dstray(*,600) le dstbby,nk)
nk = nk
bbyns = fltarr(scans,nk)
czcns = fltarr(scans,nk)
.run
for nn = 0,scans-1 do begin
   bbyns(nn,*) = bby_n0hray(keep,nn)
   czcns(nn,*) = czc_n0hray(keep,nn)
endfor
end


biol = fltarr(scans)
clut = fltarr(scans)
icec = fltarr(scans)
drys = fltarr(scans)
wets = fltarr(scans)
ltrn = fltarr(scans)
hvrn = fltarr(scans)
bigd = fltarr(scans)
grap = fltarr(scans)
hail = fltarr(scans)
unkn = fltarr(scans)
nope = fltarr(scans)
.run
for nn = 0,scans-1 do begin
if n_elements(where(bbyns(nn,*) eq 10)) ne 1 then  biol(nn) = n_elements(where(bbyns(nn,*) eq 10))/float(nk)
if n_elements(where(bbyns(nn,*) eq 20)) ne 1 then  clut(nn) = n_elements(where(bbyns(nn,*) eq 20))/float(nk)
if n_elements(where(bbyns(nn,*) eq 30)) ne 1 then  icec(nn) = n_elements(where(bbyns(nn,*) eq 30))/float(nk)
if n_elements(where(bbyns(nn,*) eq 40)) ne 1 then  drys(nn) = n_elements(where(bbyns(nn,*) eq 40))/float(nk)
if n_elements(where(bbyns(nn,*) eq 50)) ne 1 then  wets(nn) = n_elements(where(bbyns(nn,*) eq 50))/float(nk)
if n_elements(where(bbyns(nn,*) eq 60)) ne 1 then  ltrn(nn) = n_elements(where(bbyns(nn,*) eq 60))/float(nk)
if n_elements(where(bbyns(nn,*) eq 70)) ne 1 then  hvrn(nn) = n_elements(where(bbyns(nn,*) eq 70))/float(nk)
if n_elements(where(bbyns(nn,*) eq 80)) ne 1 then  bigd(nn) = n_elements(where(bbyns(nn,*) eq 80))/float(nk)
if n_elements(where(bbyns(nn,*) eq 90)) ne 1 then  grap(nn) = n_elements(where(bbyns(nn,*) eq 90))/float(nk)
if n_elements(where(bbyns(nn,*) eq 100)) ne 1 then  hail(nn) = n_elements(where(bbyns(nn,*) eq 100))/float(nk)
if n_elements(where(bbyns(nn,*) eq 140)) ne 1 then  unkn(nn) = n_elements(where(bbyns(nn,*) eq 140))/float(nk)
if n_elements(where(bbyns(nn,*) ne bbyns(nn,*))) ne 1 then  nope(nn) = n_elements(where(bbyns(nn,*) ne bbyns(nn,*)))/float(nk)
endfor
end


rain = ltrn
rain(where(ltrn/(1-nope) lt 0)) = 0
dsno = drys
dsno(where(drys/(1-nope) lt 0)) = 0
grau = grap
grau(where(grap/(1-nope) lt 0)) = 0
none = nope
none(where(nope gt .9)) = 0
unkw = nope
unkw(where(unkn/(1-nope) lt 0)) = 0
biog = biol
biog(where(biol/(1-nope) lt 0)) = 0



finals = ltrn*0
finals(where(nope gt .9)) = -20
finals(where(unkw gt 0)) = -10
finals(where(biog gt 0)) = 10
finals(where(rain gt 0)) = 60 
finals(where(dsno gt 0)) = 40
finals(where(dsno gt 0 and rain gt 0)) = 100

plot,finals,psym=2,yrange=[-30,110]
oplot,n0hczc,psym=1,color=200


good = where(n0hczc eq 40)

plot,finals(good),psym=2,yrange=[-30,110]
oplot,n0hbby(good)-3,psym=1,color=200
oplot,bbyns(good,-20)+3,psym=2,color=200
oplot,bbyns(good,-2)+6,psym=1



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

good = where(n0hczc eq 30 or n0hczc eq 40 or n0hczc eq 50,nn)
ysss = fltarr(scans)
yssr = fltarr(scans)
yssa = fltarr(scans)
.run
for n = 0,nn-1 do begin
   tmp = good(n)
   a = where(bbyns(tmp,*) eq 30 or bbyns(tmp,*) eq 40 or bbyns(tmp,*) eq 50,nsnow)
   b = where(bbyns(tmp,*) eq 60 or bbyns(tmp,*) eq 70 or bbyns(tmp,*) eq 80,nrain)
   c = where(bbyns(tmp,*) gt 20 and bbyns(tmp,*) lt 90,nprec)  
   ysss(tmp) = nsnow
   yssr(tmp) = nrain
   yssa(tmp) = nprec
endfor
end
good2 = where(n0hczc eq 60 or n0hczc eq 70 or n0hczc eq 80,nn)
ysss2 = fltarr(scans)
yssr2 = fltarr(scans)
yssa2 = fltarr(scans)
.run
for n = 0,nn-1 do begin
   tmp = good2(n)
   a = where(bbyns(tmp,*) eq 30 or bbyns(tmp,*) eq 40 or bbyns(tmp,*) eq 50,nsnow)
   b = where(bbyns(tmp,*) eq 60 or bbyns(tmp,*) eq 70 or bbyns(tmp,*) eq 80,nrain)
   c = where(bbyns(tmp,*) gt 20 and bbyns(tmp,*) lt 90,nprec)  
   ysss2(tmp) = nsnow
   yssr2(tmp) = nrain
   yssa2(tmp) = nprec
endfor
end


p.erase
p = plot(indgen(scans)*.03721,float(yssr2)/(yssa2)*100.,linestyle=6,symbol='o',sym_filled=1,yrange=[-5,105],xrange=[0,48],/current,color='red',ytitle='Percentage (%)',font_size=11,SYM_SIZE=1,transparency=50)
p2 = plot(indgen(scans)*.03721,float(yssr)/(yssa)*100.,linestyle=6,symbol='o',sym_filled=1,overplot=p,color='blue',SYM_SIZE=1,transparency=50)

p.axes[0].hide=1
xaxis2.delete
xaxis2 = axis('x',axis_range=[0,48],major=9,target=p,location=-5,minor=5,tickfont_size=11,tickname=['3/5 0Z','3/5 6Z','3/5 12Z','3/5 18Z','3/6 0Z','3/6 6Z','3/6 12Z','3/6 18Z','3/7 0Z'],ticklen=.05,subticklen=.5)


pp = plot(indgen(scans)*.03721,(yssa+yssa2)/44.*100.,overplot=p,color='gray',transparency=60)
pp0 = plot(indgen(scans)*.03721,fltarr(scans)-5,overplot=p,color='black')
pp5 = plot(indgen(scans),fltarr(scans)+50,overplot=p,color='gray',linestyle=2)
;pp2 = plot(indgen(scans)*.03721,yssa2/44.*100.,overplot=p,color='red',transparency=50)
;pp = plot(indgen(scans)*.03721,yssa/44.*100.,overplot=p,color='blue',transparency=50)

p.save,resolution=300,'/home/fcannon/classification.png'


;save,/all,file='/data/downloaded/Observations/GPM/NEXRAD/n0h_bby_20160306.IDL'

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; BRING IN REFLECTIVITY
; IS THERE MORE SNOW DURING HEAVIER PRECIP?
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
restore,'/data/downloaded/Observations/GPM/NEXRAD/n0h_bby_20160306.IDL'
fnm2 = fnm
.run
for nn = 0,1288 do begin
   tmp = fnm(nn)
   tmp2 = strsplit(tmp,'_',/extract)
   fyer = strmid(tmp2(2),0,4)
   fymn = strmid(tmp2(2),4,2)
   fydy = strmid(tmp2(2),6,2)
   fyhr = strmid(tmp2(3),0,2)
   fymm = strmid(tmp2(3),2,2)
   fysc = strmid(tmp2(3),4,2)

   dy = fydy-5.
   tim = dy*86400.+fyhr*3600.+fymm*60+fysc
   fnm2(nn) = tim
endfor
end
fnm2 = float(fnm2)


restore,'/data/downloaded/Observations/GPM/NEXRAD/ref_bby_20160306_v2.IDL'
fnm1 = fnm
.run
for nn = 0,653 do begin
   tmp = fnm(nn)
   tmp2 = strsplit(tmp,'_',/extract)
   fyer = strmid(tmp2(2),0,4)
   fymn = strmid(tmp2(2),4,2)
   fydy = strmid(tmp2(2),6,2)
   fyhr = strmid(tmp2(3),0,2)
   fymm = strmid(tmp2(3),2,2)
   fysc = strmid(tmp2(3),4,2)

   dy = fydy-5.
   tim = dy*86400.+fyhr*3600.+fymm*60+fysc
   fnm1(nn) = tim
endfor
end
fnm1 = float(fnm1)
bbyref = reform(cref(1:-1,0,0,*))

fnm1 = fnm
.run
for nn = 0,653 do begin
   tmp = fnm(nn)
   tmp2 = strsplit(tmp,'_',/extract)
   fyer = strmid(tmp2(2),0,4)
   fymn = strmid(tmp2(2),4,2)
   fydy = strmid(tmp2(2),6,2)
   fyhr = strmid(tmp2(3),0,2)
   fymm = strmid(tmp2(3),2,2)
   fysc = strmid(tmp2(3),4,2)

   dy = fydy-5.
   tim = dy*86400.+fyhr*3600.+fymm*60+fysc
   fnm1(nn) = tim
endfor
end
fnm1 = float(fnm1)
czcref = reform(cref(1:-1,1,0,*))
scans = 1290
bbref = fltarr(45,1290)
ccref = fltarr(45,1290)
.run
for nn = 0,1289 do begin
   tmp = fnm2(nn)
   near = min(abs(fnm1(0:-2)-tmp),id)
   bbref(*,nn) = bbyref(*,id)/2.-33
   ;ccref(*,nn) = fltarr(45)+czcref(id)/2.-33
   ccref(*,nn) = czcref(*,id)/2.-33
endfor
end
mbref = bbref & mbref(where(bbref lt 0)) = !values.f_nan
;nope = where(yssa+yssa2 eq 0) & mbref(*,nope) = !values.f_nan
mbref = max(mbref,dimension=1,/nan)
mcref = ccref & mcref(where(ccref lt 0)) = !values.f_nan
;nope = where(yssa+yssa2 eq 0) & mcref(*,nope) = !values.f_nan
mcref = max(mcref,dimension=1,/nan)

hgt = fltarr(1290)
.run
for nn = 0,1289 do begin
hgt(nn) = n_elements(where(bbref(*,nn) gt 0))
endfor
end

plot,(yssa2+yssa)/46.*100.,mbref,psym=2
plot,(yssr)/46.*100.,mbref,psym=2
oplot,(ysss)/46.*100.,mbref,psym=2,color=200
oplot,(yssa2+yssa)/46.*100.,mbref,psym=2




p.erase
p = plot(indgen(scans)*.03721,float(yssa2+yssa)/46.*100.,linestyle=6,symbol='o',sym_filled=1,yrange=[-5,105],xrange=[0,48],/current,color='red',ytitle='Percentage (%)',font_size=11,SYM_SIZE=1,transparency=50)
p22 = plot(indgen(scans)*.03721,float(yssr2)/(yssa2)*100.,linestyle=6,symbol='o',sym_filled=1,overplot=p,color='gray',SYM_SIZE=1,transparency=50)
p2 = plot(indgen(scans)*.03721,float(yssr)/(yssa)*100.,linestyle=6,symbol='o',sym_filled=1,overplot=p,color='blue',SYM_SIZE=1,transparency=50)
p3 = plot(indgen(scans)*.03721,mbref*5.,overplot=p,color='black',transparency=50,linestyle=6,symbol='_')

p.axes[0].hide=1
xaxis2.delete
xaxis2 = axis('x',axis_range=[0,48],major=9,target=p,location=-5,minor=5,tickfont_size=11,tickname=['3/5 0Z','3/5 6Z','3/5 12Z','3/5 18Z','3/6 0Z','3/6 6Z','3/6 12Z','3/6 18Z','3/7 0Z'],ticklen=.05,subticklen=.5)




p.erase
p = plot(float(yssa2+yssa)/46.*100.,mbref,linestyle=6,symbol='o',sym_filled=1,yrange=[0,25],xrange=[0,100],/current,color='gray',xtitle='Percentage of BBY Gates w/ Precipitation (%)',ytitle='Max Reflectivity (dBZ) at BBY Gates',font_size=11,SYM_SIZE=1,transparency=50,title='Max dBZ at BBY Gates by % of Precipitating Gates')
good = where(mbref eq mbref)
lines = linfit(float(yssa2(good)+yssa(good))/46.*100.,mbref(good),yfit=yfit2,covar=covar2)
p = plot(float(yssa2(good)+yssa(good))/46.*100.,yfit2,color='black',thick=2,overplot=p)
t = text(60,5,'r = 0.65',/data,overplot=p)
p.save,resolution=300,'/home/fcannon/dBZ_PrecipGates_BBY.png'
correlate(float(yssa2(good)+yssa(good))/46.*100.,mbref(good))








;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SAVE A TEXT FILE FOR ANDY

timearr = strarr(scans)
.run
for nn = 0,scans-1 do begin
   timearr(nn) = strmid(fnm(nn),65,15)
endfor
end
 
fracsnowsnow = ysss/yssa
fracrainsnow = yssr/yssa
fracprecsnow = yssa/45.
fracsnowrain = ysss2/yssa2
fracrainrain = yssr2/yssa2
fracprecrain = yssa2/45.
ref_bby      = bbref
ref_czc      = ccref


heads = string(['Time,%BBY Rain During CZC Snow,%BBY Snow During CZC Snow,%BBY Prec During CZC Snow,%BBY Rain During CZC Rain,%BBY Snow During CZC Rain,%BBY Prec During CZC Rain','Reflectivity at BBY (each gate)','Reflectivity at CZC'])
IDL> write_csv,'/home/fcannon/hydrometeor_class.txt',timearr(0:-2),fracrainsnow(0:-2),fracsnowsnow(0:-2),fracprecsnow(0:-2),fracrainrain(0:-2),fracsnowrain(0:-2),fracprecrain(0:-2),table_header=heads


heads = string(['Gate1,Gate2,Gate3,Gate4,Gate5,Gate6,Gate7,Gate8,Gate9,Gate10,Gate11,Gate12,Gate13,Gate14,Gate15,Gate16,Gate17,Gate18,Gate19,Gate20,Gate21,Gate22,Gate23,Gate24,Gate25,Gate26,Gate27,Gate28,Gate29,Gate30,Gate31,Gate32,Gate33,Gate34,Gate35,Gate36,Gate37,Gate38,Gate39,Gate40,Gate41,Gate42,Gate43,Gate44,Gate45'])
write_csv,'/home/fcannon/reflectivity_bby.txt',ref_bby(*,0:-2),table_header=(heads)

heads = string(['Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1,Gate1'])
write_csv,'/home/fcannon/reflectivity_czc.txt',ref_czc(*,0:-2),table_header=(heads)

heads = string(['Time'])
write_csv,'/home/fcannon/reflectivity_times.txt',timearr(0:-2),table_header=(heads)



bbyns2 = fltarr(nk,scans)
czcns2 = fltarr(nk,scans)
.run
for nn = 0,scans-1 do begin
   bbyns2(*,nn) = bby_n0hray(keep(0:-1),nn)
   czcns2(*,nn) = czc_n0hray(keep(0:-1),nn)
endfor
end


heads = string(['Gate1,Gate2,Gate3,Gate4,Gate5,Gate6,Gate7,Gate8,Gate9,Gate10,Gate11,Gate12,Gate13,Gate14,Gate15,Gate16,Gate17,Gate18,Gate19,Gate20,Gate21,Gate22,Gate23,Gate24,Gate25,Gate26,Gate27,Gate28,Gate29,Gate30,Gate31,Gate32,Gate33,Gate34,Gate35,Gate36,Gate37,Gate38,Gate39,Gate40,Gate41,Gate42,Gate43,Gate44,Gate45'])
write_csv,'/home/fcannon/h_class_czc.txt',czcns2,table_header=(heads)

heads = string(['Gate1,Gate2,Gate3,Gate4,Gate5,Gate6,Gate7,Gate8,Gate9,Gate10,Gate11,Gate12,Gate13,Gate14,Gate15,Gate16,Gate17,Gate18,Gate19,Gate20,Gate21,Gate22,Gate23,Gate24,Gate25,Gate26,Gate27,Gate28,Gate29,Gate30,Gate31,Gate32,Gate33,Gate34,Gate35,Gate36,Gate37,Gate38,Gate39,Gate40,Gate41,Gate42,Gate43,Gate44,Gate45'])
write_csv,'/home/fcannon/reflectivity_czc.txt',ref_czc,table_header=(heads)





reffs = bbref*0.
reffs(where(bbyns2 eq 60 or bbyns2 eq 70 or bbyns2 eq 80)) = bbref(where(bbyns2 eq 60 or bbyns2 eq 70 or bbyns2 eq 80))
reffs(where(reffs le 0)) = !values.f_nan
good = where(reffs eq reffs)


reffs = bbref*0.
reffs(where(bbyns2 eq 30 or bbyns2 eq 40 or bbyns2 eq 50)) = bbref(where(bbyns2 eq 30 or bbyns2 eq 40 or bbyns2 eq 50))
reffs(where(reffs le 0)) = !values.f_nan
good2 = where(reffs eq reffs)














ccref2 = ccref & ccref2(where(ccref2 lt 0)) = !values.f_nan
bbref2 = bbref & bbref2(where(bbref2 lt 0)) = !values.f_nan

.run
for nn = 0,14 do begin
   print,mean(ccref2(where(transpose(czcns2) eq nn*10)),/nan),n_elements(where(czcns2 eq nn*10)),' CZC '
   print,mean(bbref2(where(transpose(bbyns2) eq nn*10)),/nan),n_elements(where(bbyns2 eq nn*10)),' BBY '
   print,'    '
endfor
end




nn=1290
ysssb = fltarr(scans)
yssrb = fltarr(scans)
yssab = fltarr(scans)
ysssc = fltarr(scans)
yssrc = fltarr(scans)
yssac = fltarr(scans)
.run
for nn = 0,1289 do begin
   a = where(bbyns(nn,*) eq 30 or bbyns(nn,*) eq 40 or bbyns(nn,*) eq 50,nsnow)
   b = where(bbyns(nn,*) eq 60 or bbyns(nn,*) eq 70 or bbyns(nn,*) eq 80,nrain)
   c = where(bbyns(nn,*) gt 20 and bbyns(nn,*) lt 90,nprec)  
   ysssb(nn) = nsnow
   yssrb(nn) = nrain
   yssab(nn) = nprec
   tmp = indgen(scans)
   aa = where(czcns(nn,*) eq 30 or czcns(nn,*) eq 40 or czcns(nn,*) eq 50,nsnow)
   bb = where(czcns(nn,*) eq 60 or czcns(nn,*) eq 70 or czcns(nn,*) eq 80,nrain)
   cc = where(czcns(nn,*) gt 20 and czcns(nn,*) lt 90,nprec)  
   ysssc(nn) = nsnow
   yssrc(nn) = nrain
   yssac(nn) = nprec
endfor
end

brainp = yssrb/yssab & brainp(where(yssab eq 0)) = !values.f_nan
plot,brainp,psym=2,yrange=[-.05,1.05]
crainp = yssrc/yssac & crainp(where(yssac eq 0)) = !values.f_nan
oplot,crainp,psym=2,color=200
